==> 4.7.4.tar.gz: terraform-provider-decort/internal/service/cloudbroker/node/flattens.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Sergey Kisil, <svkisil@digitalenergy.online>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package node

import (
	"strconv"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/node"
)

func flattenNode(d *schema.ResourceData, item *node.RecordNode) {
	log.Debugf("flattenNode: decoded node id %d", d.Get("node_id").(int))

	d.Set("consumption", flattenConsumption(item.Consumption))
	d.Set("cpu_info", flattenCpuInfo(item.CpuInfo))
	d.Set("cpu_allocation_ratio", item.CPUAllocationRatio)
	d.Set("gid", item.GID)
	d.Set("ipaddr", item.IPAddr)
	d.Set("isolated_cpus", flattenNodeItem(item.IsolatedCpus))
	d.Set("name", item.Name)
	d.Set("need_reboot", item.NeedReboot)
	d.Set("nic_info", flattenNicInfo(item.NicInfo))
	d.Set("numa_topology", flattenNumaTopology(item.NumaTopology))
	d.Set("reserved_cpus", flattenNodeItem(item.ReservedCPUs))
	d.Set("roles", item.Roles)
	d.Set("sriov_enabled", item.SriovEnabled)
	d.Set("stack_id", item.StackID)
	d.Set("status", item.Status)
	d.Set("version", item.Version)
}

func flattenConsumption(info node.ConsumptionInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 1)
	tempRes := map[string]interface{}{
		"hostname": info.Hostname,
	}

	tempConsumed := map[string]interface{}{
		"ram":      info.Consumed.RAM,
		"computes": info.Consumed.Computes,
		"routers":  info.Consumed.Routers,
		"vcpu":     info.Consumed.VCPU,
	}
	tempRes["consumed"] = []map[string]interface{}{
		tempConsumed,
	}

	tempFree := map[string]interface{}{
		"ram": info.Free.RAM,
	}
	tempRes["free"] = []map[string]interface{}{
		tempFree,
	}

	tempReserved := map[string]interface{}{
		"ram": info.Reserved.RAM,
	}
	tempRes["reserved"] = []map[string]interface{}{
		tempReserved,
	}

	tempTotal := map[string]interface{}{
		"ram": info.Total.RAM,
	}
	tempRes["total"] = []map[string]interface{}{
		tempTotal,
	}

	res[0] = tempRes
	return res
}

func flattenNodeList(nodes *node.ListNodes) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(nodes.Data))
	for _, item := range nodes.Data {
		temp := map[string]interface{}{
			"additional_pkgs": flattenNodeItem(item.AdditionalPkgs),
			"cpu_info":        flattenCpuInfo(item.CpuInfo),
			"description":     item.Description,
			"gid":             item.GID,
			"guid":            item.GUID,
			"hostkey":         item.HostKey,
			"node_id":         item.ID,
			"ipaddr":          item.IPAddr,
			"isolated_cpus":   flattenNodeItem(item.IsolatedCpus),
			"lastcheck":       item.LastCheck,
			"machine_guid":    item.MachineGUID,
			"mainboard_sn":    item.MainboardSN,
			"memory":          item.Memory,
			"milestones":      item.Milestones,
			"model":           item.Model,
			"name":            item.Name,
			"need_reboot":     item.NeedReboot,
			"net_addr":        flattenNetAddr(item.NetAddr),
			"network_mode":    item.NetworkMode,
			"nic_info":        flattenNicInfo(item.NicInfo),
			"node_uuid":       item.NodeUUID,
			"numa_topology":   flattenNumaTopology(item.NumaTopology),
			"peer_backup":     item.PeerBackup,
			"peer_log":        item.PeerLog,
			"peer_stats":      item.PeerStats,
			"pgpus":           item.Pgpus,
			"public_keys":     item.PublicKeys,
			"release":         item.Release,
			"reserved_cpus":   flattenNodeItem(item.ReservedCPUs),
			"roles":           item.Roles,
			"seps":            item.Seps,
			"serial_num":      item.SerialNum,
			"sriov_enabled":   item.SriovEnabled,
			"stack_id":        item.StackID,
			"status":          item.Status,
			"tags":            item.Tags,
			"type":            item.Type,
			"version":         item.Version,
		}
		res = append(res, temp)
	}
	return res
}

func flattenNumaTopology(info node.NumaTopologyInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 1)
	tempRes := map[string]interface{}{
		"node_num": info.NodeNum,
	}
	resNodes := make([]map[string]interface{}, 0, len(info.Nodes))
	for _, item := range info.Nodes {
		memoryTemp := []map[string]interface{}{
			{
				"one_g": item.Memory.OneG,
				"two_m": item.Memory.TwoM,
				"total": item.Memory.Total,
			},
		}
		temp := map[string]interface{}{
			"cpu_list": item.CPUList,
			"memory":   memoryTemp,
		}
		resNodes = append(resNodes, temp)
	}
	tempRes["nodes"] = resNodes
	res[0] = tempRes
	return res
}

func flattenNicInfo(infos node.ListNicInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(infos))
	for _, item := range infos {
		temp := map[string]interface{}{
			"driver":    item.Driver,
			"max_vfs":   item.MaxVFS,
			"numa_node": item.NumaNode,
			"num_vfs":   item.NumVFS,
			"os_name":   item.OSName,
			"pci_slot":  item.PCISlot,
			"vf_list":   flattenVFList(item.VFList),
		}
		res = append(res, temp)
	}
	return res
}

func flattenVFList(vfList []interface{}) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(vfList))
	for _, v := range vfList {
		vConv := v.(map[string]interface{})
		temp := map[string]interface{}{
			"fn_id":    vConv["fnId"],
			"pci_slot": vConv["pciSlot"],
		}
		res = append(res, temp)
	}
	return res
}

func flattenNetAddr(adresses node.ListNetAddr) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(adresses))
	for _, item := range adresses {
		temp := map[string]interface{}{
			"cidr":  item.CIDR,
			"index": item.Index,
			"ip":    item.IP,
			"mac":   item.Mac,
			"mtu":   item.MTU,
			"name":  item.Name,
		}
		res = append(res, temp)
	}
	return res
}

func flattenCpuInfo(info node.CpuInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 1)
	temp := map[string]interface{}{
		"clock_speed": info.ClockSpeed,
		"core_count":  info.CoreCount,
		"phys_count":  info.PhysCount,
	}
	res[0] = temp
	return res
}

func flattenNodeItem(m []interface{}) []string {
	output := []string{}
	for _, item := range m {
		switch d := item.(type) {
		case string:
			output = append(output, d)
		case int:
			output = append(output, strconv.Itoa(d))
		case int64:
			output = append(output, strconv.FormatInt(d, 10))
		case float64:
			output = append(output, strconv.FormatInt(int64(d), 10))
		default:
			output = append(output, "")
		}
	}
	return output
}

==> 4.8.0.tar.gz: terraform-provider-decort/internal/service/cloudbroker/node/flattens.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Sergey Kisil, <svkisil@digitalenergy.online>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package node

import (
	"strconv"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/node"
)

func flattenNode(d *schema.ResourceData, item *node.RecordNode) {
	log.Debugf("flattenNode: decoded node id %d", d.Get("node_id").(int))

	d.Set("consumption", flattenConsumption(item.Consumption))
	d.Set("cpu_info", flattenCpuInfo(item.CpuInfo))
	d.Set("cpu_allocation_ratio", item.CPUAllocationRatio)
	d.Set("gid", item.GID)
	d.Set("ipaddr", item.IPAddr)
	d.Set("isolated_cpus", flattenNodeItem(item.IsolatedCpus))
	d.Set("name", item.Name)
	d.Set("need_reboot", item.NeedReboot)
	d.Set("net_addr", flattenGetNetAddr(item.NetAddr))
	d.Set("nic_info", flattenNicInfo(item.NicInfo))
	d.Set("numa_topology", flattenNumaTopology(item.NumaTopology))
	d.Set("reserved_cpus", flattenNodeItem(item.ReservedCPUs))
	d.Set("roles", item.Roles)
	d.Set("sriov_enabled", item.SriovEnabled)
	d.Set("stack_id", item.StackID)
	d.Set("status", item.Status)
	d.Set("version", item.Version)
}

func flattenConsumption(info node.ConsumptionInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 1)
	tempRes := map[string]interface{}{
		"hostname": info.Hostname,
	}

	tempConsumed := map[string]interface{}{
		"ram":      info.Consumed.RAM,
		"computes": info.Consumed.Computes,
		"routers":  info.Consumed.Routers,
		"vcpu":     info.Consumed.VCPU,
	}
	tempRes["consumed"] = []map[string]interface{}{
		tempConsumed,
	}

	tempFree := map[string]interface{}{
		"ram": info.Free.RAM,
	}
	tempRes["free"] = []map[string]interface{}{
		tempFree,
	}

	tempReserved := map[string]interface{}{
		"ram": info.Reserved.RAM,
	}
	tempRes["reserved"] = []map[string]interface{}{
		tempReserved,
	}

	tempTotal := map[string]interface{}{
		"ram": info.Total.RAM,
	}
	tempRes["total"] = []map[string]interface{}{
		tempTotal,
	}

	res[0] = tempRes
	return res
}

func flattenNodeList(nodes *node.ListNodes) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(nodes.Data))
	for _, item := range nodes.Data {
		temp := map[string]interface{}{
			"additional_pkgs": flattenNodeItem(item.AdditionalPkgs),
			"cpu_info":        flattenCpuInfo(item.CpuInfo),
			"description":     item.Description,
			"gid":             item.GID,
			"guid":            item.GUID,
			"hostkey":         item.HostKey,
			"node_id":         item.ID,
			"ipaddr":          item.IPAddr,
			"isolated_cpus":   flattenNodeItem(item.IsolatedCpus),
			"lastcheck":       item.LastCheck,
			"machine_guid":    item.MachineGUID,
			"mainboard_sn":    item.MainboardSN,
			"memory":          item.Memory,
			"milestones":      item.Milestones,
			"model":           item.Model,
			"name":            item.Name,
			"need_reboot":     item.NeedReboot,
			"net_addr":        flattenNetAddr(item.NetAddr),
			"network_mode":    item.NetworkMode,
			"nic_info":        flattenNicInfo(item.NicInfo),
			"node_uuid":       item.NodeUUID,
			"numa_topology":   flattenNumaTopology(item.NumaTopology),
			"peer_backup":     item.PeerBackup,
			"peer_log":        item.PeerLog,
			"peer_stats":      item.PeerStats,
			"pgpus":           item.Pgpus,
			"public_keys":     item.PublicKeys,
			"release":         item.Release,
			"reserved_cpus":   flattenNodeItem(item.ReservedCPUs),
			"roles":           item.Roles,
			"seps":            item.Seps,
			"serial_num":      item.SerialNum,
			"sriov_enabled":   item.SriovEnabled,
			"stack_id":        item.StackID,
			"status":          item.Status,
			"tags":            item.Tags,
			"type":            item.Type,
			"version":         item.Version,
		}
		res = append(res, temp)
	}
	return res
}

func flattenNumaTopology(info node.NumaTopologyInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 1)
	tempRes := map[string]interface{}{
		"node_num": info.NodeNum,
	}
	resNodes := make([]map[string]interface{}, 0, len(info.Nodes))
	for _, item := range info.Nodes {
		memoryTemp := []map[string]interface{}{
			{
				"one_g": item.Memory.OneG,
				"two_m": item.Memory.TwoM,
				"total": item.Memory.Total,
			},
		}
		temp := map[string]interface{}{
			"cpu_list": item.CPUList,
			"memory":   memoryTemp,
		}
		resNodes = append(resNodes, temp)
	}
	tempRes["nodes"] = resNodes
	res[0] = tempRes
	return res
}

func flattenNicInfo(infos node.ListNicInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(infos))
	for _, item := range infos {
		temp := map[string]interface{}{
			"driver":    item.Driver,
			"max_vfs":   item.MaxVFS,
			"numa_node": item.NumaNode,
			"num_vfs":   item.NumVFS,
			"os_name":   item.OSName,
			"pci_slot":  item.PCISlot,
			"vf_list":   flattenVFList(item.VFList),
		}
		res = append(res, temp)
	}
	return res
}

func flattenVFList(vfList []interface{}) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(vfList))
	for _, v := range vfList {
		vConv := v.(map[string]interface{})
		temp := map[string]interface{}{
			"fn_id":    vConv["fnId"],
			"pci_slot": vConv["pciSlot"],
		}
		res = append(res, temp)
	}
	return res
}

func flattenNetAddr(addresses node.ListNetAddr) []map[string]interface{} {
	res := make([]map[string]interface{}, 0, len(addresses))
	for _, item := range addresses {
		temp := map[string]interface{}{
			"cidr":  item.CIDR,
			"index": item.Index,
			"ip":    item.IP,
			"mac":   item.Mac,
			"mtu":   item.MTU,
			"name":  item.Name,
		}
		res = append(res, temp)
	}
	return res
}

func flattenGetNetAddr(address node.NetAddr) []map[string]interface{} {
	res := make([]map[string]interface{}, 1)
	temp := map[string]interface{}{
		"ip":   address.IP,
		"name": address.Name,
	}
	res[0] = temp
	return res
}

func flattenCpuInfo(info node.CpuInfo) []map[string]interface{} {
	res := make([]map[string]interface{}, 1)
	temp := map[string]interface{}{
		"clock_speed": info.ClockSpeed,
		"core_count":  info.CoreCount,
		"phys_count":  info.PhysCount,
	}
	res[0] = temp
	return res
}

func flattenNodeItem(m []interface{}) []string {
	output := []string{}
	for _, item := range m {
		switch d := item.(type) {
		case string:
			output = append(output, d)
		case int:
			output = append(output, strconv.Itoa(d))
		case int64:
			output = append(output, strconv.FormatInt(d, 10))
		case float64:
			output = append(output, strconv.FormatInt(int64(d), 10))
		default:
			output = append(output, "")
		}
	}
	return output
}
